--Rant Layout
--Design based on p3lim's oUF layout.

local _G = getfenv(0)
local RantTooltip = _G.RantTooltip

if not RantTooltip then return end

local font = "Interface\\AddOns\\RantTooltip_Layout\\media\\Charcoal.ttf"
local barTexture = "Interface\\AddOns\\RantTooltip_Layout\\media\\minimalist"

RantTooltip.ReactionColors[2] = {1, 0, 0}
RantTooltip.ReactionColors[4] = {1, 1, 0}
RantTooltip.ReactionColors[5] = {0, 1, 0}

RantTooltip.PowerColors["MANA"] = {0, 144/255, 1}

local backdrop = {
	bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
	insets = { top = -1, bottom = -1, left = -1, right = -1 }
}

RantTooltip.Tags["$npc"] = function(unit) 
	return (not UnitIsPlayer(unit)) and RantTooltip:RGBHex(GetQuestDifficultyColor(UnitLevel(unit))).."(NPC)|r" 
end

local layout = function(self)
	self:SetBackdrop(backdrop)
	self:SetScale(1)
	self:SetBackdropColor(0, 0, 0)
	
	self.bg = self.bg or self:CreateTexture(nil, "BORDER")
	self.bg:SetAllPoints(self)
	self.bg:SetTexture(barTexture)
	self.bg:SetVertexColor(0.3, 0.3, 0.3)
	
	if self == GameTooltip then
		RantTooltip:SetInitialAnchor("CURSOR", 0, 1)
		--Default anchor
		--RantTooltip:SetInitialAnchor("BOTTOMRIGHT", -13, 65)
		
		--Characters inside { brackets will only show if all variables encompassed by the brackets have a return value
		RantTooltip:SetUnitTags{
			"$dnd$name{ ($aserver)}{: $target}",
			"{[$guild]}",
			"$level$classification {$race $class}{$type $npc}",
			"$misc",
		}
		
		GameTooltipHeaderText:SetFont(font, 14, "OUTLINE")
		GameTooltipText:SetFont(font, 13, "OUTLINE")
		
		self.raidIcon = self.raidIcon or self:CreateTexture(nil, "ARTWORK")
		self.raidIcon:SetWidth(16); self.raidIcon:SetHeight(16)
		self.raidIcon:SetPoint("TOP", self, 0, 8)
		
		self.combatIcon = self.combatIcon or self:CreateTexture(nil, "OVERLAY")
		self.combatIcon:SetWidth(20); self.combatIcon:SetHeight(20)
		self.combatIcon:SetTexture("Interface\\CharacterFrame\\UI-StateIcon")
		self.combatIcon:SetTexCoord(0.5, 1.0, 0, 0.5)
		self.combatIcon:SetPoint("TOPRIGHT", self, "TOPLEFT", 2, 2)
		self.combatIcon:Hide()
		
		self.PostOnShow = function(self)
			if self.doodad then
				self:SetBackdropColor(0, 0, 0)
			end
		end
		
		self.PostOnHide = function(self)
			self:SetBackdropColor(0, 0, 0)
		end
		
		self.PostSetUnit = function(_, unit)
			GameTooltipTextLeft1:SetTextColor(GameTooltip_UnitColor(unit))
		end
		
		for _, name in ipairs{"Health", "Power"} do
			local statusBar = CreateFrame("StatusBar", nil, self)
			statusBar:SetBackdrop(backdrop)
			statusBar:SetBackdropColor(0, 0, 0)
			statusBar:SetStatusBarTexture(barTexture)
			statusBar:SetStatusBarColor(0.25, 0.25, 0.35)
			
			statusBar.bg = statusBar.bg or statusBar:CreateTexture(nil, "BORDER")
			statusBar.bg:SetAllPoints(statusBar)
			
			if name == "Health" then
				statusBar.Anchor = { 0, 15 }
				statusBar.bg:SetTexture(0.3, 0.3, 0.3)
			
				statusBar.text = statusBar.text or statusBar:CreateFontString(nil, "OVERLAY")
				statusBar.text:SetFont(font, 12, "OUTLINE")
				statusBar.text:SetJustifyH("CENTER")
				statusBar.text:SetAllPoints(statusBar)
				statusBar.Tags = "{$miss }$perc%"
				
				statusBar:SetHeight(12)
				statusBar:SetPoint("TOPLEFT", self, "BOTTOMLEFT", 0, -3)
				statusBar:SetPoint("TOPRIGHT", self, "BOTTOMRIGHT", 0, -3)
			else
				statusBar.Anchor = { 0, 20 }
				statusBar.unitColors = true
				statusBar.powerColors = false
				
				statusBar.bg:SetTexture("Interface\\ChatFrame\\ChatFrameBackground")
				statusBar.bg.color = 0.3
				
				statusBar:SetHeight(4)
				statusBar:SetPoint("TOPLEFT", self.HealthBar, "BOTTOMLEFT", 0, -1)
				statusBar:SetPoint("TOPRIGHT", self.HealthBar, "BOTTOMRIGHT", 0, -1)
			end
			
			statusBar:Hide()
			self[name.."Bar"] = statusBar
		end
	end
	
	if self == ItemRefTooltip then
		self.itemIcon = self:CreateTexture(nil, "OVERLAY")
		self.itemIcon:SetWidth(37); self.itemIcon:SetHeight(37)
		self.itemIcon:SetPoint("TOPRIGHT", self, "TOPLEFT", -2, 0)
	end
end

RantTooltip:RegisterLayout("Rant", layout)
RantTooltip:UseLayout("Rant")