﻿aSettings = CreateFrame("Frame", "aSettings", UIParent)
aSettings:SetScript("OnEvent", function(self, event, ...) self[event](self, event, ...) end)
aSettings:RegisterEvent("PLAYER_DEAD")
aSettings:RegisterEvent("ADDON_LOADED")
aSettings:RegisterEvent("PLAYER_LOGIN")
aSettings:RegisterEvent("MERCHANT_SHOW")
aSettings:RegisterEvent("DUEL_REQUESTED")
aSettings:RegisterEvent("START_LOOT_ROLL")
aSettings:RegisterEvent("PARTY_INVITE_REQUEST")

local defaults = {
    AutoGreedOnGreen = false,
    AutoRepair = true,
    UseGuildBank = true,
    SellGreyCrap = true,
    DeclineDuels = true,
    AcceptInvites = true,
    NoErrors = true,
    UIScale = false,
    AutoRez = true,
}

--[[ Loading ]]
function aSettings:ADDON_LOADED(event, addon)
    if(addon~="aSettings") then return end

    aSettingsDB = setmetatable(aSettingsDB or {}, {__index = defaults})
    
    CreateFrame("Frame", nil, InterfaceOptionsFrame):SetScript("OnShow", function()
        if(not IsAddOnLoaded("aSettings_Config")) then LoadAddOn("aSettings_Config") end
        self:SetScript("OnShow", nil)
    end)

    SLASH_ASETTINGS1 = "/asettings"
    SLASH_ASETTINGS2 = "/aset"
    SlashCmdList["ASETTINGS"] = function()
        if(not IsAddOnLoaded("aSettings_Config")) then LoadAddOn("aSettings_Config") end
        InterfaceOptionsFrame_OpenToCategory("aSettings") 
    end

    if(aSettingsDB.NoErrors==true) then UIErrorsFrame:UnregisterEvent("UI_ERROR_MESSAGE") end

    SetCVar("screenshotQuality", 6)
    SetCVar("cameraDistanceMax", 50)
    SetCVar("cameraDistanceMaxFactor", 2)

    self:UnregisterEvent(event)
end

--[[ Creating info text ]]
local frame = CreateFrame("Frame", nil, UIParent)
frame:SetScript("OnUpdate", FadingFrame_OnUpdate)
frame.fadeInTime = 0.5
frame.fadeOutTime = 2
frame.holdTime = 3
frame:Hide()

local text = frame:CreateFontString("aInfoText", "OVERLAY")
text:SetFont("Fonts\\FRIZQT__.TTF", 32, "THICKOUTLINE")
text:SetPoint("CENTER", UIParent, "CENTER")
text:SetTextColor(0.41, 0.8, 0.94)

aInfoText_ShowText = function(s)
    text:SetText(s)
    FadingFrame_Show(frame)
end

--[[ Greed on greens ]]
function aSettings:START_LOOT_ROLL(event, id)
    if(not aSettingsDB.AutoGreedOnGreen) then return end
    if(id and select(4, GetLootRollItemInfo(id))==2) then
        RollOnLoot(id, 2)
    end
end

--[[ Repair & sell greys ]]
function aSettings:MERCHANT_SHOW()
    if(aSettingsDB.AutoRepair==true and CanMerchantRepair()) then
        local cost = GetRepairAllCost()
        if(not cost or cost==0) then return end

        local CanGuildRepair = aSettingsDB.UseGuildBank and IsInGuild() and CanGuildBankRepair() and GetGuildBankWithdrawMoney()>cost and GetGuildBankMoney()>cost
        if(CanGuildRepair) then
            RepairAllItems(1)
            aInfoText_ShowText(format("Guild bank repair: %.1fg", cost * 0.0001))
        elseif(GetMoney()>cost) then
            RepairAllItems()
            aInfoText_ShowText(format("Repair cost: %.1fg", cost * 0.0001))
        end
    end

    if(aSettingsDB.SellGreyCrap==true) then
        local bag, slot 
        for bag = 0, 4 do
            for slot = 0, GetContainerNumSlots(bag) do
                local link = GetContainerItemLink(bag, slot)
                if link and (select(3, GetItemInfo(link))==0) then
                    UseContainerItem(bag, slot)
                end
            end
        end
    end
end

--[[ UIScale ]]
local Scales = {
    ["768"] = 1,
    ["800"] = 0.96,
    ["864"] = 0.88888888888888,
    ["900"] = 0.85333333333333,
    ["960"] = 0.8,
    ["1024"] = 0.75,
    ["1050"] = 0.73142857142857,
    ["1080"] = 0.71111111111111,
    ["1200"] = 0.64,
}

function aSettings:PLAYER_LOGIN()
    if(not aSettingsDB.UIScale) then return end
    local _, height = string.match((({GetScreenResolutions()})[GetCurrentResolution()] or ""), "(%d+).-(%d+)")
    if(height and Scales[height]) then
        SetCVar("useUiScale", 1)
        SetCVar("uiScale", Scales[height])
    else
        SetCVar("useUiScale", 0)
        if(tonumber(height)>768) then
            print("Your resolution is not yet supported, disabling UI Scale.")
        end
    end
end

--[[ Auto decline duels ]]
function aSettings:DUEL_REQUESTED(event, name)
    if(not aSettingsDB.DeclineDuels) then return end
    HideUIPanel(StaticPopup1)
    CancelDuel()
    aInfoText_ShowText("Declined duel request from: "..name)
end

--[[ Auto rez ]]
function aSettings:PLAYER_DEAD()
    if(not aSettingsDB.AutoRez) then return end
    if(MiniMapBattlefieldFrame.status and MiniMapBattlefieldFrame.status=="active") then
        RepopMe()
    end
end

--[[ Some slash commands ]]
SlashCmdList["FRAME"] = function() print(GetMouseFocus():GetName()) end
SLASH_FRAME1 = "/gn"
SLASH_FRAME2 = "/frame"

SlashCmdList["GETPARENT"] = function() print(GetMouseFocus():GetParent():GetName()) end
SLASH_GETPARENT1 = "/gp"
SLASH_GETPARENT2 = "/parent"

SlashCmdList["RELOADUI"] = function() ReloadUI() end
SLASH_RELOADUI1 = "/rl"
SLASH_RELOADUI2 = "/кд"
SLASH_RELOADUI3 = "/пи"

SlashCmdList["RCSLASH"] = function() DoReadyCheck() end
SLASH_RCSLASH1 = "/rc"
SLASH_RCSLASH2 = "/кс"
SLASH_RCSLASH2 = "/рч"
SLASH_RCSLASH2 = "/пг"

SlashCmdList["TICKET"] = function() ToggleHelpFrame() end
SLASH_TICKET1 = "/ticket"
SLASH_TICKET2 = "/gm"
SLASH_TICKET3 = "/гм"

SlashCmdList["DISABLE_ADDON"] = function(s) DisableAddOn(s) end
SLASH_DISABLE_ADDON1 = "/dis"

SlashCmdList["ENABLE_ADDON"] = function(s) EnableAddOn(s) end
SLASH_ENABLE_ADDON1 = "/en"

--[[ Accept invites ]]
local IsFriend = function(name)
    for i=1, GetNumFriends() do if(GetFriendInfo(i)==name) then return true end end
    if(IsInGuild()) then for i=1, GetNumGuildMembers() do if(GetGuildRosterInfo(i)==name) then return true end end end
end

function aSettings:PARTY_INVITE_REQUEST(event, name)
    if(not aSettingsDB.AcceptInvites) then return end
    if(IsFriend(name)) then
        aInfoText_ShowText("Accepted invite from: "..name)
        AcceptGroup()
        for i = 1, 4 do
            local frame = _G["StaticPopup"..i]
            if(frame:IsVisible() and frame.which=="PARTY_INVITE") then
                frame.inviteAccepted = 1
                StaticPopup_Hide("PARTY_INVITE")
                return
            end
        end
    else
        SendWho(name)
    end
end

--[[ Set oRA2 tanks! ]]
SLASH_SETORATANK1 = "/oramt"
SLASH_SETORATANK2 = "/setmt"
SLASH_SETORATANK3 = "/omt"
SLASH_SETORATANK4 = "/щкфье"
SlashCmdList["SETORATANK"] = function(msg)
    local a, b = strsplit(" ", msg)
    if(type(tonumber(a))=="number" and b) then
        SendAddonMessage("oRA", "SET " .. a .. " " .. b, "RAID")
    elseif(a=="remove" or a=="сдуфк" and b) then
        SendAddonMessage("oRA", "R "..b, "RAID")
    else
        print("|cff00ffffaSettings|r: how to set oRA2 tanks.\n/oramt |cff00ffffindex |cff00ff00name|r - set a player with |cff00ff00name|r as tank. Index must be a number (1-10).\n/oramt |cff00ffffremove |cff00ff00name|r - remove a tank.")
    end
end

--[[ We like mounts! ]]
local DALARAN, UNDERBELLY, KRASUS, WINTERGRASP, VIOLET
local locale = GetLocale()
if(locale=="ruRU") then
	DALARAN = "Даларан"
	UNDERBELLY = "Клоака"
	KRASUS = "Площадка Краса"
	WINTERGRASP = "Озеро Ледяных Оков"
    VIOLET = "Аметистовая цитадель"
elseif(locale=="frFR") then
	DALARAN = "Dalaran"
	UNDERBELLY = "Les Entrailles"
	KRASUS = "Aire de Krasus"
	WINTERGRASP = "Joug-d'hiver"
    VIOLET = "Citadelle Pourpre"
elseif(locale=="deDE") then
    DALARAN = "Dalaran" 
    UNDERBELLY = "Die Schattenseite"
    KRASUS = "Krasus' Landeplatz" 
    WINTERGRASP = "Tausendwintersee"
    VIOLET = "Die Violette Zitadelle"
--[[ Template for other locales
elseif(locale=="") then
	DALARAN = ""
	UNDERBELLY = ""
	KRASUS = ""
	WINTERGRASP = ""
    VIOLET = ""
]]
else
	DALARAN = "Dalaran" 
	UNDERBELLY = "The Underbelly"
	KRASUS = "Krasus' Landing" 
	WINTERGRASP = "Wintergrasp"
    VIOLET = "The Violet Citadel"
end

function Mountz(groundmount, flyingmount)
    local num = GetNumCompanions("MOUNT")
    if not num or IsMounted() then
        Dismount()
        return
    end
    if CanExitVehicle() then 
        VehicleExit()
        return
    end
    local flyablex = (IsFlyableArea() and (not (GetZoneText()==DALARAN or GetZoneText()==WINTERGRASP) or GetSubZoneText()==KRASUS or GetSubZoneText()==UNDERBELLY or GetSubZoneText()==VIOLET) and (UnitLevel("player")>76 or GetCurrentMapContinent()==3))
    if IsAltKeyDown() then
        flyablex = not flyablex
    end
    for i=1, num, 1 do
        local _, info = GetCompanionInfo("MOUNT", i)
        if flyingmount and info == flyingmount and flyablex then
            CallCompanion("MOUNT", i)
            return
        elseif groundmount and info == groundmount and not flyablex then
            CallCompanion("MOUNT", i)
            return
        end
    end
end

